"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@checkup/core");
const semver_1 = require("semver");
const ESLINT_CONFIG = {
    parser: 'babel-eslint',
    parserOptions: {
        ecmaVersion: 2018,
        sourceType: 'module',
        ecmaFeatures: {
            legacyDecorators: true,
        },
    },
    rules: {
        strict: ['error', 'never'],
    },
    useEslintrc: false,
    allowInlineConfig: false,
    ignore: false,
};
class ValidateEsmPackageTask extends core_1.BaseValidationTask {
    constructor(pluginName, context) {
        super(pluginName, context);
        this.taskName = 'validate-esm-package';
        this.taskDisplayName = 'Validate ESM Package';
        this.description = "Validates whether a project's package.json uses valid esm properties";
        this.category = 'validation';
        this.addRule({
            properties: {
                component: {
                    name: 'validation',
                },
            },
        });
        this.addValidationSteps();
    }
    addValidationSteps() {
        let pkg = this.context.pkg;
        this.addValidationStep('Contains "type" property set to "module" in package.json', () => {
            return {
                isValid: !!(pkg.type && pkg.type === 'module'),
            };
        });
        this.addValidationStep('Does not contain a "main" property in package.json', () => {
            return {
                isValid: !!pkg.main,
            };
        });
        this.addValidationStep('Contains an "exports" property set to a valid value in package.json', () => {
            let exports = pkg.exports;
            return {
                isValid: !!(exports &&
                    (typeof exports === 'string' ||
                        (Array.isArray(exports) && exports.every((value) => typeof value === 'string')) ||
                        (typeof exports === 'object' && exports !== null))),
            };
        });
        this.addValidationStep('Requires at least Node 12 in the "engines" property of package.json', () => {
            // Node 13.2.0 was the first Node version that shipped with ESM without the --experimental-module flag.
            return {
                isValid: !!(pkg.engines &&
                    pkg.engines['node'] &&
                    semver_1.default.satisfies('13.2.0', pkg.engines['node'])),
            };
        });
        this.addValidationStep('Should not have "use strict" in any JavaScript files', async () => {
            let analyzer = new core_1.ESLintAnalyzer(ESLINT_CONFIG);
            let { results } = await analyzer.analyze(this.context.paths.filterByGlob('**/*.js'));
            return {
                isValid: results.length === 0,
            };
        });
    }
    async run() {
        let validationResults = await this.validate();
        for (let [messageText, validationResult] of validationResults) {
            this.addValidationResult(messageText, validationResult.isValid);
        }
        return this.results;
    }
}
exports.default = ValidateEsmPackageTask;
//# sourceMappingURL=validate-esm-package-task.js.map